-- $Id: db-schema.sql,v 1.1 2002/01/04 13:26:42 fpmip Exp $

-- Author : Tycho Fruru <tycho@fruru.com>
-- Copyright (C) 2001 Tycho Fruru <tycho@fruru.com>
-- Copyright (C) 2001 Conostix S.A. Luxembourg - Tycho Fruru <tycho@conostix.com>

-- This program is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2 of the License, or (at your
-- option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-- for more details.


\c template1
drop database ipfc;
create database ipfc;
\c ipfc

-- users give us the possibility of having several sites with grouped
-- agents

create table users 
(
	login_name 	text primary key,
	fullname	text,
	creation_date	timestamp default now(),
	start_date	timestamp default now(),
	end_date	timestamp default '9999-01-01',
	user_enabled	boolean default 't'
);

-- a list of all understood agent types
create table agent_types
(
	type_name 	text unique,	-- name to use to determine table name
	name		text		-- freeform type description
);

-- Every agent is defined in table agents
-- 
-- agent_url is the url which describes the base directory where that agent's
--     information can be found
-- agent_name is a free-form name, hopefully descriptive for the agent

create table agents
(
	agent_id 	serial primary key,
	agent_type_name text constraint existing_agent_type 
			    references agent_types(type_name)
			    not null,
	agent_name 	text,
	agent_location text,
	agent_enabled	boolean default 't'
);

create table agent_group_descriptions
(
	group_id	int primary key,
	group_description text
);

create table agent_groups
(
	group_id	int not null
			    constraint existing_group_id
			    	references agent_group_descriptions(group_id),
	agent_id	int constraint existing_agent
			    references agents(agent_id)
			    not null
);

create table agent_url_ids
(
	agent_id	int constraint existing_agent
			    references agents(agent_id)
			    not null,
	agent_url_id	text not null
);

create table user_rights
(
	login_name	text constraint existing_login_name
				    references users(login_name)
			    not null,
	agent_id	int constraint existing_agent
			    references agents(agent_id)
			    not null,
	can_view	boolean not null
				default 'f',
	can_manage	boolean not null
				default 'f',
	can_annotate	boolean not null
				default 'f',
	can_purgelogs	boolean not null
				default 'f',
	user_enabled	boolean not null
				default 't'
);

-- Table to manage table names for the different event/alert/status categories
create table table_names
(
	agent_type_name text
			    references agent_types(type_name)
			    not null,
	entry_type	 text
			    not null,
	entry_subtype	 text
			    not null,
	entry_version	 int
			    default 0
			    not null,
	entry_transport	 text
			    not null,
	table_name	 text
			    not null,
	transport_function_name
			 text
			    not null, -- name of the function to decapsulate the transport encoding
	function_name	 text
			    not null, -- name of the function to process 1 entry of this format
	has_src_addr	 boolean not null
			    default 'f',
	has_src_port	 boolean not null
			    default 'f',
	has_dst_addr	 boolean not null
			    default 'f',
	has_dst_port	 boolean not null
			    default 'f',	
			    
	unique(agent_type_name,entry_type,entry_subtype,entry_version)
);

create unique index table_names_unique on 
	table_names(agent_type_name, entry_type, entry_subtype, entry_version);

-- Table to manage logfiles (to see whether logfiles are complete)
create table logfiles
(
	logfile_id	serial8 primary key,
	agent_id	int 
			    constraint existing_agent
			    references agents(agent_id)
		   	    not null, 	
	wrapper_id	smallint default 0 not null,	-- number of the wrapper sending events for this agent_id
	generation_id	text not null, 		-- should be increased every time module wrapper is restarted (eg. unix time)
	sequence_id	int not null, 	-- should be increased every time new logfile is sent
	transac_id	text,			-- internal transaction ID, to please Alex ;-]
	agent_time	timestamp,		-- date @ agent
	read_time	timestamp default now(),
	
	unique(agent_id,wrapper_id,generation_id,sequence_id)
);


create table events
(
	event_id 	serial8 primary key,
	agent_id 	int 
			    constraint existing_agent 
			    references agents(agent_id)
			    not null,
	event_type	text,			-- type of message (status, event, alert, ...)
	event_subtype	text,			-- exactly what subtype of event we're logging (error_log, access_log, ...)
	event_version	int
			    default 1,		-- more details can be found in events_<event_type>_<event_version>
	time 		timestamp
			    default now(),
	logfile_id	int8
			    constraint existing_logfile
			    references logfiles(logfile_id)
			    not null,
	functional_severity	int,
	security_severity	int,
	message		text			-- a (agent type independent) message
);

create index events_time_index on events(time);
create index events_agent_id_index on events(agent_id);

create table events_with_src_host
(
	src_host	text
) inherits (events);

create table events_with_src_host_port
(
	ip_proto	int default 0,
	src_port	int
) inherits (events_with_src_host);

create table events_with_src_dst_host
(
	dst_host	text
) inherits (events_with_src_host);

create table events_with_src_dst_host_port
(
	dst_host	text,
	dst_port	int
) inherits (events_with_src_host_port);

create table events_firewall
(
	firewall_host	text,
	in_interface	text,
	out_interface	text,
	accept		boolean,
	drop		boolean,
	reject		boolean
) inherits (events_with_src_dst_host_port);

create table transport_syslog
(
	syslog_date	text,
	host		text,
	logging_program	text,
	pid		text
);

create table event_snort_alert_1
(
	snort_dump	bytea
) inherits (events_with_src_dst_host_port);

create table apache_access_log_1
(
	virtual_host	text,
	remote_logname	text default '-',	-- from ident (almost not used anymore ...)
	remote_username	text default '-',	-- from http authentication
	url		text,
	status		int,
	bytes_sent	int,
	referer		text,
	user_agent	text,
	foreign key (agent_id) references agents(agent_id),
	foreign key (logfile_id) references logfiles(logfile_id)		
) inherits (events_with_src_host);

create index aal1lei on apache_access_log_1(event_id);
create index aal1lt  on apache_access_log_1(time);
create index aal1ai  on apache_access_log_1(agent_id);
create index aal1sh  on apache_access_log_1(src_host);

create table apache_error_log_1
(
) inherits (events_with_src_host);

create index ael1ei on apache_error_log_1(event_id);
create index ael1t  on apache_error_log_1(time);
create index ael1ai on apache_error_log_1(agent_id);

create table chkp_log41_1
(
	rule		int,
	info		text,
	encrypt		boolean,
	remote_username	text,
	ip_length	int,
	foreign key (agent_id) references agents(agent_id),
	foreign key (logfile_id) references logfiles(logfile_id)	
) inherits (events_firewall);

create table netfilter_log_1
(
	prefix		text,
	in_mac_addr	text,
	length		int,
	ip_tos		int,
	ip_prec		int,
	ip_ttl		int,
	ip_id		int,
	ip_ce		boolean default 'f',
	ip_df		boolean default 'f',
	ip_mf		boolean default 'f',
	ip_frag		int,
	ip_opt		text,
	incomplete_hdr  boolean,
	tcp_seq		int8,	-- int is not good for seq and ack (they are unsigned !)
	tcp_ackseq	int8,
	tcp_window	int,
	tcp_reserved_bits text,
	tcp_cwr		boolean,
	tcp_ece		boolean,
	tcp_urg		boolean,
	tcp_ack		boolean,
	tcp_psh		boolean,
	tcp_rst		boolean,
	tcp_syn		boolean,
	tcp_fin		boolean,
	tcp_urgp	int8,
	tcp_opt		text,
	udp_length	int,
	icmp_type	int,
	icmp_code	int,
	ipsec_spi	text
) inherits (events_firewall);

create table syslog_1_syslog
(
	log_message	text,
	foreign key (agent_id) references agents(agent_id),
	foreign key (logfile_id) references logfiles(logfile_id)
) inherits (events,transport_syslog);

create index se1id on syslog_1_syslog(event_id);
create index se1t  on syslog_1_syslog(time);
create index se1ai on syslog_1_syslog(agent_id);
create index se1lm on syslog_1_syslog(log_message);
	
create table availability_1
(
	retry_time	int,	-- number of seconds before we expect another file
	foreign key (agent_id) references agents(agent_id),
	foreign key (logfile_id) references logfiles(logfile_id)
) inherits (events);

create index a1ei on availability_1(event_id);
create index a1t  on availability_1(time);
create index a1ai on availability_1(agent_id);

create table transport_mon
(
	alert_type	text,
	mon_group	text,
	service		text,
	summary		text,
	members		text
);

create table mon_alert_1
(
) inherits (transport_mon,events);

create index ma1id on mon_alert_1(event_id);
create index ma1t  on mon_alert_1(time);
create index ma1ai on mon_alert_1(agent_id);

-- dns resolution mapping
create table host_addresses
(
	event_id	int8,
	host_address	inet,
	host_name	text
);

create index haei on host_addresses(event_id);
